//+------------------------------------------------------------------+
//|                                              SlopeTradeADX.mq4 |
//|                                   Copyright  2008, Randy Knight |
//|                                           knight.randy@gmail.com |
//+------------------------------------------------------------------+
#property copyright "Copyright  2008, Randy Knight"
#property link      "knight.randy@gmail.com"
#include <stdlib.mqh>

//---- input parameters
extern string     sec1 = "---- EA Config ------------";
extern string     CommentString = "SlopeTradeADX";
extern bool       Verbose = true;
extern bool       TakeTrades = true; 
extern bool       SignalMail = true;
extern bool       AlertOn = false;
extern bool       ExitOnChange = true;
extern bool       UseMM = true;
extern bool       MarginCheck = true;
extern int        MaxOpenTrades = 10;
extern bool       EveryTickMode = true;

extern string     sec5 = "------ MM -------------";
extern string     Method = "Dollar";  //must be "Percent" or "Dollar"
extern double     DollarRiskPerTrade = 20;
extern double     PercentRiskPerTrade = 3;

extern string     sec2 = "--------- Signal Settings ---------- ";
extern bool       UseSlopeTF1 = true;
extern int        SlopeTF1 = 1440;
extern bool       UseSlopeTF2 = true;
extern int        SlopeTF2 = 240;
extern bool       UseSlopeTF3 = true;
extern int        SlopeTF3 = 60;
extern bool       UseSlopeTF4 = true;
extern int        SlopeTF4 = 30;
extern int        ADXTF = 30;
extern double     ADXLevel = 35;

extern string     sec3 = " ---------- Trade Management --------";
extern double     Lots = 1; //ignored  if UseMM = true
extern double     Slippage = 10;
extern int        StopLoss = 50;
extern int        TakeProfit = 0;   
extern int        BreakEvenGain = 50;
extern int        BreakEvenLevel = 10;
extern int        TrailGain = 100;
extern int        TrailLevel = 50;

extern string     sec4 = "----------- Indicator Settings ---------";
extern string     SettingsFile = "SlopeSettings.txt";
extern int        SlopePeriod = 80;
extern int        SlopeMethod = 3;
extern int        SlopePrice = 0;



int Ticket;

int init()
{
   string ExistTrade = ExistTrade(Symbol());
   string InitStr;
   
   if (ExistTrade != "none")
   {
      TakeTrades = false;
   }
}


int start()
{
   
   int Trend1 =-1, Trend2= -1, Trend3 = -1, Trend4 = -1;
   double ADX;
   
   //bail if not newbar
   if (EveryTickMode == false && NewBar() == false)
      return (0);

   //validate MM Method
   if ((UseMM == true) && (Method != "Dollar" && Method != "Percent"))
   {
      Alert("Method must be \"Percent\" or \"Dollar\"");
      return (0);      
   }

   //Get SlopeDirecton for all timeframes
   if (UseSlopeTF1 == true)
      Trend1 = SlopeDirection(SlopeTF1, SlopePeriod, SlopeMethod, SlopePrice);
      
   if (UseSlopeTF2 == true)
      Trend2 = SlopeDirection(SlopeTF2, SlopePeriod, SlopeMethod, SlopePrice);
            
   if (UseSlopeTF3 == true)
      Trend3 = SlopeDirection(SlopeTF3, SlopePeriod, SlopeMethod, SlopePrice);
            
   if (UseSlopeTF4 == true)
      Trend4 = SlopeDirection(SlopeTF4, SlopePeriod, SlopeMethod, SlopePrice);
            
   //Get ADX 
   if (ADXTF > 0)
      ADX = iADX(Symbol(),ADXTF,14,PRICE_CLOSE,MODE_MAIN,0);
   else
      ADX = 100;      
   
  //Get Signal
  string Signal = CalcSignal(Trend1, Trend2, Trend3, Trend4);
  
      
   //verbose output
   if (Verbose == true)      
      Comment("     Broker time= ",TimeToStr(TimeCurrent()),
         "\n","     Signal= ",(Signal),
         "\n","     ADX= ",(ADX),
         "\n","     TakeTrades = ",(TakeTrades));
         
   //journal testing
   if (IsTesting() == true)
   {
      Print("Signal: " + Signal
         + " |ADX: " + ADX 
         + " |ADXLevel: " + ADXLevel);
   }      
   
   //existing trade 
   string ExistTrade = ExistTrade(Symbol());
   
   //ExitOnChange
   if (ExitOnChange == true && Signal != "none" && ExistTrade != Signal && ExistTrade != "none")
   {
      double ClosePrice;
      color CloseColor;
      
      if (ExistTrade == "long")
      {
         ClosePrice = Bid;
         CloseColor = Red;
      }
      else
      {
         ClosePrice = Ask;
         CloseColor = Green;
      }
      
      OrderSelect(Ticket,SELECT_BY_TICKET,MODE_TRADES);
      
      if (OrderClose(Ticket,OrderLots(),ClosePrice,Slippage,CloseColor) != true)
      {
         Print("Error closing order : ", ErrorDescription(GetLastError()));
         Alert("Error closing order : ", ErrorDescription(GetLastError()));
      } 
   }
   
   
   //Get Pips
   int Pips;
   if (ExistTrade != "none")
      Pips = GetPips(Symbol());
      
   //TrailStop
   if (ExistTrade != "none")
   {
      AdjustStop(Symbol());
   }
   
   //Place trade
   if (ExistTrade == "none" && TakeTrades == true && Signal != "none" && OpenTrades() < MaxOpenTrades && ADX >= ADXLevel)
   {
      double StopLossLevel, TakeProfitLevel;
      int Dir;
      
      StopLossLevel = CalcStopLevel(Symbol(), Signal);
      TakeProfitLevel = CalcTPLevel(Symbol(), Signal);
    
      if (Signal == "long")
         Dir = OP_BUY;
      else
         Dir = OP_SELL;
         
      int NewTicket = OrderPlace(Dir, StopLossLevel, TakeProfitLevel, true);
      
      if (NewTicket > 0)
      {
         TakeTrades = false;
         Alert(Symbol() + ": TakeTrades = " + TakeTrades);
      }         
   }
      
   //Send mail
   if (SignalMail == true && (NewTicket > 0 || (ExistTrade == "none" && Signal != "none" && NewTicket == 0 && ADX >= ADXLevel)))
   {
      if (NewTicket > 0)
         SendMail("[" + CommentString + "]", "[" + Symbol() + "] Ticket#: " + DoubleToStr(Ticket,0));
   }
   
   //Alert
   if (AlertOn == true && (NewTicket > 0 || (ExistTrade == "none" && Signal != "none" && NewTicket == 0 && ADX >= ADXLevel)))
   {
      if (NewTicket > 0)
         Alert ("[" + CommentString + "]", "[" + Symbol() + "] Ticket#: " + DoubleToStr(NewTicket,0));
   }
   
   
   return(0);
}


bool NewBar() {

	static datetime LastTime = 0;

	if (Time[0] != LastTime) {
		LastTime = Time[0];		
		return (true);
	} else
		return (false);
}


string CalcSignal(int Trend1, int Trend2, int Trend3, int Trend4)
{
   //Calculate Slope Signal
    string Signal = "none";
   /*
      -1 = ignore
      0  = none
      1  = short 
      2  = long 
   */  

   
   //all 4 used
   if (UseSlopeTF1 == true && UseSlopeTF2 == true && UseSlopeTF3 == true && UseSlopeTF4 == true)
   {
      //Print ("All four");
      if (Trend1 == 1 && Trend2 == 1 && Trend3 == 1 && Trend4 == 1)
         Signal = "short";
      else if (Trend1 == 2 && Trend2 == 2 && Trend3 == 2 && Trend4 == 2)
         Signal = "long";

      return (Signal);         
   }
   
   //3 used
   if (UseSlopeTF1 == true && UseSlopeTF2 == true && UseSlopeTF3 == true && UseSlopeTF4 == false)
   {
      Print ("Three");
      if (Trend1 == 1 && Trend2 == 1 && Trend3 == 1)
         Signal = "short";
      else if (Trend1 == 2 && Trend2 == 2 && Trend3 == 2)
         Signal = "long";

      return (Signal);         
   }
   

   //2 used
   if (UseSlopeTF1 == true && UseSlopeTF2 == true && UseSlopeTF3 == false && UseSlopeTF4 == false)
   {
   Print ("Two");
      if (Trend1 == 1 && Trend2 == 1)
         Signal = "short";
      else if (Trend1 == 2 && Trend2 == 2)
         Signal = "long";
         
      return(Signal);         
   }
    
   //1 used
   if (UseSlopeTF1 == true && UseSlopeTF2 == false && UseSlopeTF3 == false && UseSlopeTF4 == false)
   {
      Print ("One");
      if (Trend1 == 1)
         Signal = "short";
      else if (Trend1 == 2)
         Signal = "long";

      return(Signal);
   }
}   


string AdjustStop(string Pair)
{
   int Pips, PipsFromStop;
   double NewStop;
   
   //calculate new stop   
   if (OrderType() == OP_BUY)
      {
         //stop has never been moved
         if (OrderStopLoss() < OrderOpenPrice())
         {
            Pips = GetPips(Symbol());
            
            //pips over BreakEvenGain
            if(Pips > BreakEvenGain) 
            {
                  NewStop = OrderOpenPrice() + (BreakEvenLevel * MarketInfo(Pair,MODE_POINT));
            }               
         }
         else
         {
            //trail stop higher
            PipsFromStop = (MarketInfo(Pair,MODE_BID) - OrderStopLoss()) / MarketInfo(Pair,MODE_POINT);
            
         
            if (PipsFromStop > TrailGain)
            {
               //set stop to TrailLevel
               NewStop = MarketInfo(Pair,MODE_BID) - (TrailLevel * MarketInfo(Pair,MODE_POINT));
               if (NewStop < OrderStopLoss())
                  NewStop = 0;
            }
         }   
      }
   else
      {
         //stop has never been moved
         if (OrderStopLoss() > OrderOpenPrice())
         {
            Pips = GetPips(Symbol());
            
            //pips over BreakEvenGain
            if(Pips > BreakEvenGain) 
            {
                  NewStop = OrderOpenPrice() - (BreakEvenLevel * MarketInfo(Pair,MODE_POINT));
            }               
         }
         else
         {
            //trail stop higher
            PipsFromStop = (OrderStopLoss() - MarketInfo(Pair,MODE_ASK)) / MarketInfo(Pair,MODE_POINT);
         
            if (PipsFromStop > TrailGain)
            {
               //set stop to TrailLevel
               NewStop = MarketInfo(Pair,MODE_ASK) + (TrailLevel * MarketInfo(Pair,MODE_POINT));
               if (NewStop > OrderStopLoss())
                  NewStop = 0;
            }
         }   
      }
      
    //modify Order
    if ((NewStop != OrderStopLoss()) && (NewStop != 0))
    {
      OrderModify(OrderTicket(),OrderOpenPrice(),NewStop,OrderTakeProfit(),0);               
      Print ("Stop moved to " + NewStop + " for Ticket#: " + OrderTicket());
    }      
}



double CalcTPLevel(string Pair, string Direction)
{
   double TPLevel;
   
   if (TakeProfit == 0)
      return (0);
      
   if (Direction == "short")
      TPLevel = MarketInfo(Pair,MODE_BID) - (TakeProfit * MarketInfo(Pair, MODE_POINT));
   else        
      TPLevel = MarketInfo(Pair,MODE_ASK) + (TakeProfit * MarketInfo(Pair, MODE_POINT));
      
   return (TPLevel);      
}

double CalcStopLevel(string Pair, string Direction)
{

   double StopLossLevel;
   
   if (StopLoss == 0)
      return (0);
      
   if (Direction == "short")
      StopLossLevel = MarketInfo(Pair,MODE_ASK) + (StopLoss * MarketInfo(Pair, MODE_POINT));
   else        
      StopLossLevel = MarketInfo(Pair,MODE_BID) - (StopLoss * MarketInfo(Pair, MODE_POINT));
      
   return (StopLossLevel);      
}



int GetPips(string Pair)
{
   int pips;
   if (OrderType() == OP_BUY)
   {
      //get profit
      pips = (MarketInfo(Pair,MODE_ASK) - OrderOpenPrice()) / MarketInfo(Pair,MODE_POINT);
   }  
   
   if (OrderType() == OP_SELL)
   {
      //get profit
      pips = (OrderOpenPrice() - MarketInfo(Pair,MODE_BID)) / MarketInfo(Pair,MODE_POINT);
   }      
   
   return (pips);
}     

string ExistTrade(string Pair)
{
   int Total = OrdersTotal();
   string Exist = "none";
   
   // find open or pending orders for this symbol
   for(int pos = 0; pos <Total; pos++)
   {
      if(OrderSelect(pos,SELECT_BY_POS,MODE_TRADES)==false) 
         continue;
         
      if (OrderSymbol() == Pair && OrderComment() == CommentString)
      {
         if (OrderType() == OP_BUY)
            Exist = "long";
         else  
            Exist = "short";
            
         Ticket = OrderTicket();            
         
         break;

      }         
   }
   
   return (Exist);
}  

int OpenTrades()
{
   int Trades = 0;
   int Total = OrdersTotal();
   
   // find all open trades 
   for(int pos = 0; pos <Total; pos++)
   {
      if(OrderSelect(pos,SELECT_BY_POS,MODE_TRADES)==false) 
         continue;
         
      if (OrderComment() == CommentString)
      {
         Trades = Trades + 1;
      }         
   }
   
   return (Trades);
}  


int OrderPlace(int OrdType, double SL, double TP, bool Arrow)
{
   double Price;
   color ArrowClr;
   
   double PipValue = MarketInfo(Symbol(),MODE_TICKVALUE);
   
   if (OrdType == OP_BUY)
   {
      Price = Ask;
      ArrowClr = Green;
   }  
   else  
   {  
      Price = Bid;
      ArrowClr = Red;
   }      
   
   //calculate lots
   if (UseMM == true)
   {
      if (Method == "Dollar")
      {
         Lots = DollarRiskPerTrade / (PipValue * StopLoss);
         Lots = StrToDouble(DoubleToStr(Lots,2));
      }
      else if (Method == "Percent")
      {
         DollarRiskPerTrade = AccountEquity() * (PercentRiskPerTrade /100);
         Lots = DollarRiskPerTrade / (PipValue * StopLoss);
         Lots = StrToDouble(DoubleToStr(Lots,2));
      }     
      else 
      {
         Alert("If UseMM=true, must specify Method");
         return (0);
      }
   }
   
   
   if (MarginCheck == true)
   {
      double FreeMargin = AccountFreeMarginCheck(Symbol(), OrdType,Lots);
      double AtRisk = StopLoss * PipValue * Lots;
      double MarginRequired = (MarketInfo(Symbol(),MODE_MARGINREQUIRED)) * Lots;
      double MarginLevel = MarginRequired + AtRisk;
      
      
      if (FreeMargin < MarginLevel)
      {
         Alert("Not enough money. Free Margin = ", AccountFreeMargin(), " Need = ", MarginLevel);
         return(0);
      }
   }

   
   int Ticket =  OrderSend(Symbol(), OrdType, Lots, Price, Slippage, SL, TP, CommentString, 0, 0, ArrowClr);
   if(Ticket > 0) 
   {
      if (OrderSelect(Ticket, SELECT_BY_TICKET, MODE_TRADES)) 
      {
         Print("Order opened : ", OrderOpenPrice());
         Alert("Order opened : ", OrderOpenPrice());
      } 
   }   
   else 
   {
      Print("Error opening order : ", ErrorDescription(GetLastError()));
      Alert("Error opening order : ", ErrorDescription(GetLastError()));
   }
   
   return(Ticket);
}

int SlopeDirection(int TF, int SlopePeriod, int SlopeMethod, int SlopePrice)
{
   int Trend =999;
   
   //calculate slope for daily
   double Buffer0 = iCustom(NULL, TF, "Slope Direction Line", SlopePeriod,SlopeMethod,SlopePrice, 0, 0);
   double Buffer1 = iCustom(NULL, TF, "Slope Direction Line", SlopePeriod,SlopeMethod,SlopePrice, 1, 0);
   
   if(Buffer0 == EMPTY_VALUE)
      Trend = 1;  //short

   if(Buffer1 == EMPTY_VALUE)
      Trend = 2;  //long
      
   if (Trend == 999 && IsTesting() == true)
      Print("Buffer0: " + Buffer0 + " | Buffer1: " + Buffer1);      

   return (Trend);
}
 





